@@ -1,6 +1,6 @@
 /*
  * noVNC: HTML5 VNC client
- * Copyright (C) 2012 Joel Martin
+ * Copyright (C) 2019 The noVNC Authors
  * Licensed under MPL 2.0 (see LICENSE.txt)
  *
  * See README.md for usage and integration instructions.
@@ -10,22 +10,24 @@
  * Logging/debug routines
  */
 
-var _log_level = 'warn';
+let _logLevel = 'warn';
 
-var Debug = function (msg) {};
-var Info = function (msg) {};
-var Warn = function (msg) {};
-var Error = function (msg) {};
+let Debug = () => {};
+let Info = () => {};
+let Warn = () => {};
+let Error = () => {};
 
-export function init_logging (level) {
+export function initLogging(level) {
     if (typeof level === 'undefined') {
-        level = _log_level;
+        level = _logLevel;
     } else {
-        _log_level = level;
+        _logLevel = level;
     }
 
-    Debug = Info = Warn = Error = function (msg) {};
+    Debug = Info = Warn = Error = () => {};
+
     if (typeof window.console !== "undefined") {
+        /* eslint-disable no-console, no-fallthrough */
         switch (level) {
             case 'debug':
                 Debug = console.debug.bind(window.console);
@@ -38,14 +40,17 @@ export function init_logging (level) {
             case 'none':
                 break;
             default:
-                throw new Error("invalid logging type '" + level + "'");
+                throw new window.Error("invalid logging type '" + level + "'");
         }
+        /* eslint-enable no-console, no-fallthrough */
     }
-};
-export function get_logging () {
-    return _log_level;
-};
+}
+
+export function getLogging() {
+    return _logLevel;
+}
+
 export { Debug, Info, Warn, Error };
 
 // Initialize logging level
-init_logging();
+initLogging();
